import nilmdb
from nilmdb.printf import *
import nilmdb.cmdline

from nose.tools import *
from nose.tools import assert_raises
import json
import itertools 
import os
import shutil
import sys
import threading
import urllib2
from urllib2 import urlopen, HTTPError
import Queue
import cStringIO
import shlex

from test_helpers import *

testdb = "tests/cmdline-testdb"

def setup_module():
    global test_server, test_db
    # Clear out DB
    recursive_unlink(testdb)

    # Start web app on a custom port
    test_db = nilmdb.NilmDB(testdb, sync = False)
    test_server = nilmdb.Server(test_db, host = "127.0.0.1",
                                port = 12380, stoppable = False,
                                fast_shutdown = True,
                                force_traceback = False)
    test_server.start(blocking = False)

def teardown_module():
    global test_server, test_db
    # Close web app
    test_server.stop()
    test_db.close()

class TestCmdline(object):

    def run(self, arg_string, input_string = ""):
        """Run a cmdline client with the specified argument string,
        passing the given input.  Returns a tuple with the output and
        exit code"""
        class stdio_wrapper:
            def __init__(self, stdin, stdout, stderr):
                self.io = (stdin, stdout, stderr)
            def __enter__(self):
                self.saved = ( sys.stdin, sys.stdout, sys.stderr )
                ( sys.stdin, sys.stdout, sys.stderr ) = self.io
            def __exit__(self, type, value, traceback):
                ( sys.stdin, sys.stdout, sys.stderr ) = self.saved
        infile = cStringIO.StringIO(input_string)
        outfile = cStringIO.StringIO()
        with stdio_wrapper(infile, outfile, outfile) as s:
            try:
                nilmdb.cmdline.Cmdline(shlex.split(arg_string)).run()
                sys.exit(0)
            except SystemExit as e:
                exitcode = e.code
        output = outfile.getvalue()
        self.output = output
        self.exitcode = exitcode

    def ok(self, arg_string, input_string = ""):
        self.run(arg_string, input_string)
        if self.exitcode != 0:
            self.dump()
            eq_(self.exitcode, 0)

    def fail(self, arg_string, input_string = ""):
        self.run(arg_string, input_string)
        if self.exitcode == 0:
            self.dump()
            ne_(self.exitcode, 0)

    def check(self, checkstring):
        in_(checkstring, self.output)

    def dump(self):
        print '-----dump start-----'
        print self.output[:-1]
        print '-----dump end-----'

    def test_cmdline_basic(self):
       
        # help
        self.ok("--help")
        self.check("usage:")

        # fail for no args
        self.fail("")

        # fail for no such option
        self.fail("--nosuchoption")

        # fail for bad command
        self.fail("badcommand")

        # try some URL constructions
        self.fail("--url http://nosuchurl/ info")
        self.check("Couldn't resolve host 'nosuchurl'")

        self.fail("--url nosuchurl info")
        self.check("Couldn't resolve host 'nosuchurl'")

        self.fail("-u nosuchurl/foo info")
        self.check("Couldn't resolve host 'nosuchurl'")

        self.fail("-u localhost:0 info")
        self.check("couldn't connect to host")

        self.ok("-u localhost:12380 info")
        self.ok("info")

    def test_cmdline_info(self):
        self.ok("info")
        self.check("Server URL: http://localhost:12380/")
        self.check("Server version: " + test_server.version)

